package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BaseDieselGeneratorEnt;
import pps.core.base.entity.BaseDieselGeneratorView;
import pps.core.base.mapper.BaseDieselGeneratorMapper;
import pps.core.base.mapper.BaseDieselGeneratorViewMapper;
import pps.core.base.service.data.base_diesel_generator.*;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XService
public class BaseDieselGeneratorService {
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseDieselGenerator(XContext context, CreateBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        BaseDieselGeneratorEnt entity = new BaseDieselGeneratorEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseDieselGenerator(XContext context, UpdateBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        QueryWrapper<BaseDieselGeneratorEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDieselGeneratorEnt::getId, input.getId());
        BaseDieselGeneratorEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseDieselGenerator(XContext context, DeleteBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        QueryWrapper<BaseDieselGeneratorEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDieselGeneratorEnt::getId, input.getId());
        BaseDieselGeneratorEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseDieselGeneratorOutput> getBaseDieselGenerator(XContext context, GetBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        QueryWrapper<BaseDieselGeneratorEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDieselGeneratorEnt::getId, input.getId());
        BaseDieselGeneratorEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseDieselGeneratorOutput output = new GetBaseDieselGeneratorOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseDieselGeneratorOutput> queryBaseDieselGenerator(XContext context, QueryBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        QueryWrapper<BaseDieselGeneratorEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BaseDieselGeneratorEnt::getId, input.getId())
                .eq(BaseDieselGeneratorEnt::getIsDeleted, input.getIsDeleted())
                .like(BaseDieselGeneratorEnt::getCreateById, input.getCreateById())
                .like(BaseDieselGeneratorEnt::getCreateByName, input.getCreateByName())
                .eq(BaseDieselGeneratorEnt::getCreateTime, input.getCreateTime())
                .like(BaseDieselGeneratorEnt::getModifyById, input.getModifyById())
                .like(BaseDieselGeneratorEnt::getModifyByName, input.getModifyByName())
                .eq(BaseDieselGeneratorEnt::getModifyTime, input.getModifyTime())
                .like(BaseDieselGeneratorEnt::getOuId, input.getOuId())
                .like(BaseDieselGeneratorEnt::getDeviceName, input.getDeviceName())
                .like(BaseDieselGeneratorEnt::getMakerNumber, input.getMakerNumber())
                .like(BaseDieselGeneratorEnt::getUnitTypeKey, input.getUnitTypeKey())
                .eq(BaseDieselGeneratorEnt::getNormalPower, input.getNormalPower())
                .eq(BaseDieselGeneratorEnt::getStandbyPower, input.getStandbyPower());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseDieselGeneratorOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseDieselGeneratorOutput> dynamicQueryBaseDieselGenerator(XContext context, CustomQueryInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        return XMapperHelper.query(mapper, input, BaseDieselGeneratorEnt.class, QueryBaseDieselGeneratorOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseDieselGeneratorViewOutput> getBaseDieselGeneratorView(XContext context, GetBaseDieselGeneratorViewInput input) {
        BaseDieselGeneratorViewMapper mapper = context.getBean(BaseDieselGeneratorViewMapper.class);
        BaseDieselGeneratorView record = new BaseDieselGeneratorView();
        XCopyUtils.copyObject(input, record);
        BaseDieselGeneratorView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseDieselGeneratorViewOutput output = new GetBaseDieselGeneratorViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseDieselGeneratorViewOutput> queryBaseDieselGeneratorView(XContext context, QueryBaseDieselGeneratorViewInput input) {
        BaseDieselGeneratorViewMapper mapper = context.getBean(BaseDieselGeneratorViewMapper.class);
        BaseDieselGeneratorView record = new BaseDieselGeneratorView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseDieselGeneratorView> list = mapper.selectList(record);
        PageInfo<BaseDieselGeneratorView> pageInfo = new PageInfo<>(list);
        List<QueryBaseDieselGeneratorViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseDieselGeneratorViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
