package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import pps.core.base.entity.BasePriceStrategyDetailEnt;
import pps.core.base.entity.BasePriceStrategyEnt;
import pps.core.base.entity.BasePriceStrategyMonthEnt;
import pps.core.base.entity.BasePriceStrategyView;
import pps.core.base.mapper.BasePriceStrategyDetailMapper;
import pps.core.base.mapper.BasePriceStrategyMapper;
import pps.core.base.mapper.BasePriceStrategyMonthMapper;
import pps.core.base.mapper.BasePriceStrategyViewMapper;
import pps.core.base.service.data.base_price_strategy.*;
import pps.core.base.service.data.base_price_strategy_month.CreateBasePriceStrategyMonthInput;
import pps.core.base.service.data.base_price_strategy_month.GetBasePriceStrategyMonthOutput;
import pps.core.base.service.data.base_price_strategy_month.UpdateBasePriceStrategyMonthInput;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 市电峰谷配置模块
 *
 * @author ZWT
 * @date 2023/08/24
 */
@XService
@XText("市电峰谷配置模块")
public class BasePriceStrategyService {

    /**
     * 市电峰谷策略--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("市电峰谷策略--新增")
    public XServiceResult createBasePriceStrategy(XContext context, CreateBasePriceStrategyInput input) {
//        PpsUserSession session = context.getSession(PpsUserSession.class);
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
        return XTransactionHelper.begin(context, () -> {
            BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
            BasePriceStrategyEnt entity = new BasePriceStrategyEnt();
            XCopyUtils.copyObject(input, entity);
            this.setBasePriceStrategyEntDefault(entity, session);
            mapper.insert(entity);
            String strategyId = entity.getId();
            List<CreateBasePriceStrategyMonthInput> inputMonths = input.getMonths();
            if (!inputMonths.isEmpty()) {
                List<BasePriceStrategyMonthEnt> monthList = new ArrayList<>(16);
                XCopyUtils.copyList(inputMonths, monthList, BasePriceStrategyMonthEnt.class);
                BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
                BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
                this.saveStrategyMonthDetail(monthMapper, detailMapper, monthList, strategyId, session);
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 市电峰谷策略--修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("市电峰谷策略--修改")
    public XServiceResult updateBasePriceStrategy(XContext context, UpdateBasePriceStrategyInput input) {
        //        PpsUserSession session = context.getSession(PpsUserSession.class);
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
        String strategyId = input.getId();
        return XTransactionHelper.begin(context, () -> {
            BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
            QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BasePriceStrategyEnt::getId, strategyId);
            BasePriceStrategyEnt entity = mapper.selectOne(queryWrapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            List<UpdateBasePriceStrategyMonthInput> inputMonths = input.getMonths();
            if (!inputMonths.isEmpty()) {
                List<BasePriceStrategyMonthEnt> monthList = new ArrayList<>(16);
                XCopyUtils.copyList(inputMonths, monthList, BasePriceStrategyMonthEnt.class);
                List<Integer> collect = monthList.stream()
                        .map(BasePriceStrategyMonthEnt::getStrategyMonth)
                        .collect(Collectors.toList());
                BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
                //删除月信息
                monthMapper.delete(new QueryWrapper<BasePriceStrategyMonthEnt>()
                        .lambda()
                        .eq(BasePriceStrategyMonthEnt::getStrategyId, strategyId)
                        .in(BasePriceStrategyMonthEnt::getStrategyMonth, collect)
                );
                BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
                //删除详情
                detailMapper.delete(new QueryWrapper<BasePriceStrategyDetailEnt>()
                        .lambda()
                        .eq(BasePriceStrategyDetailEnt::getStrategyId, strategyId)
                        .in(BasePriceStrategyDetailEnt::getStrategyMonth, collect)
                );
                this.saveStrategyMonthDetail(monthMapper, detailMapper, monthList, strategyId, session);
            }
            XCopyUtils.copyObject(input, entity);
            this.setBasePriceStrategyEntDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 市电峰谷策略--删除
     * (如果被引用,则不能删除)
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("市电峰谷策略--删除")
    public XServiceResult deleteBasePriceStrategy(XContext context, DeleteBasePriceStrategyInput input) {
        //        PpsUserSession session = context.getSession(PpsUserSession.class);
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
        String strategyId = input.getId();
        //todo: 检查是否被其它功能引用
        return XTransactionHelper.begin(context, () -> {
            BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
            QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BasePriceStrategyEnt::getId, strategyId);
            BasePriceStrategyEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //删除月数据
            BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
            monthMapper.delete(new QueryWrapper<BasePriceStrategyMonthEnt>()
                    .lambda()
                    .eq(BasePriceStrategyMonthEnt::getStrategyId, strategyId));
            //删除明细数据
            BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
            detailMapper.delete(new QueryWrapper<BasePriceStrategyDetailEnt>()
                    .lambda()
                    .eq(BasePriceStrategyDetailEnt::getStrategyId, strategyId));
            mapper.deleteById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 市电峰谷策略--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePriceStrategyOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("市电峰谷策略--详情")
    public XSingleResult<GetBasePriceStrategyOutput> getBasePriceStrategy(XContext context, GetBasePriceStrategyInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        String strategyId = input.getId();
        QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .select(BasePriceStrategyEnt::getId,
                        BasePriceStrategyEnt::getPolicyName,
                        BasePriceStrategyEnt::getAreaCode,
                        BasePriceStrategyEnt::getStrategyYear,
                        BasePriceStrategyEnt::getIsFixedPrice,
                        BasePriceStrategyEnt::getFixedElectricityPrice)
                .eq(BasePriceStrategyEnt::getId, strategyId);
        BasePriceStrategyEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePriceStrategyOutput output = new GetBasePriceStrategyOutput();
        XCopyUtils.copyObject(entity, output);
        //查月份
        BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
        List<BasePriceStrategyMonthEnt> monthEntList = monthMapper.selectList(new QueryWrapper<BasePriceStrategyMonthEnt>()
                .lambda()
                .select(BasePriceStrategyMonthEnt::getId, BasePriceStrategyMonthEnt::getStrategyMonth)
                .eq(BasePriceStrategyMonthEnt::getStrategyId, strategyId));
        if (!monthEntList.isEmpty()) {
            //查明细
            BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
            List<BasePriceStrategyDetailEnt> detailEntList = detailMapper.selectList(new QueryWrapper<BasePriceStrategyDetailEnt>()
                    .lambda()
                    .select(BasePriceStrategyDetailEnt::getStrategyMonth,
                            BasePriceStrategyDetailEnt::getPeriodTypeKey,
                            BasePriceStrategyDetailEnt::getStartTime,
                            BasePriceStrategyDetailEnt::getEndTime,
                            BasePriceStrategyDetailEnt::getElectrovalence,
                            BasePriceStrategyDetailEnt::getInternalSettlementPrice)
                    .eq(BasePriceStrategyDetailEnt::getStrategyId, strategyId));
            if (!detailEntList.isEmpty()) {
                Map<Integer, List<BasePriceStrategyDetailEnt>> collect = detailEntList.stream()
                        .collect(Collectors.groupingBy(BasePriceStrategyDetailEnt::getStrategyMonth));
                for (BasePriceStrategyMonthEnt monthEnt : monthEntList) {
                    monthEnt.setDetails(collect.get(monthEnt.getStrategyMonth()));
                }
            }
            List<GetBasePriceStrategyMonthOutput> months = new ArrayList<>(16);
            XCopyUtils.copyList(monthEntList, months, GetBasePriceStrategyMonthOutput.class);
            output.setMonths(months);
        }
        return XSingleResult.success(output);
    }

    /**
     * 市电峰谷策略--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBasePriceStrategyViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("市电峰谷策略--分页列表")
    public XPageResult<QueryBasePriceStrategyViewOutput> queryBasePriceStrategyView(XContext context, QueryBasePriceStrategyViewInput input) {
        BasePriceStrategyViewMapper mapper = context.getBean(BasePriceStrategyViewMapper.class);
        BasePriceStrategyView record = new BasePriceStrategyView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePriceStrategyView> list = mapper.selectList(record);
        PageInfo<BasePriceStrategyView> pageInfo = new PageInfo<>(list);
        List<QueryBasePriceStrategyViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePriceStrategyViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 保存策略明细
     *
     * @param monthMapper  月映射器
     * @param detailMapper 细节映射器
     * @param monthList    月清单
     * @param strategyId   战略id
     * @param session      会话
     */
    private void saveStrategyMonthDetail(BasePriceStrategyMonthMapper monthMapper, BasePriceStrategyDetailMapper detailMapper, List<BasePriceStrategyMonthEnt> monthList, String strategyId, PpsUserSession session) {
        Integer strategyMonth;
        List<BasePriceStrategyDetailEnt> detailList;
        for (BasePriceStrategyMonthEnt monthEnt : monthList) {
            this.setBasePriceStrategyMonthEntDefault(monthEnt, strategyId, session);
            monthMapper.insert(monthEnt);
            strategyMonth = monthEnt.getStrategyMonth();
            detailList = monthEnt.getDetails();
            for (BasePriceStrategyDetailEnt detailEnt : detailList) {
                this.setBasePriceStrategyDetailEntDefault(detailEnt, strategyId, strategyMonth, session);
                detailMapper.insert(detailEnt);
            }
        }
    }

    /**
     * 设置市电峰谷当月明细配置默认值
     *
     * @param entity        实体
     * @param strategyId    战略id
     * @param strategyMonth 战略月
     * @param session       会话
     */
    private void setBasePriceStrategyDetailEntDefault(BasePriceStrategyDetailEnt entity, String strategyId, Integer strategyMonth, PpsUserSession session) {
        Date now = new Date();
        entity.setStrategyId(strategyId);
        entity.setStrategyMonth(strategyMonth);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(now);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(now);
    }

    /**
     * 设置市电峰谷月配置默认值
     *
     * @param entity     实体
     * @param strategyId 战略id
     * @param session    会话
     */
    private void setBasePriceStrategyMonthEntDefault(BasePriceStrategyMonthEnt entity, String strategyId, PpsUserSession session) {
        Date now = new Date();
        entity.setStrategyId(strategyId);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(now);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(now);
    }

    /**
     * 设置市电峰谷策略默认值
     *
     * @param entity  实体
     * @param session 会话
     */
    private void setBasePriceStrategyEntDefault(BasePriceStrategyEnt entity, PpsUserSession session) {
        String id = entity.getId();
        Date now = new Date();
        String userId = session.getId();
        String userName = session.getUserName();
        if (StringUtils.isEmpty(id)) {
            entity.setCreateById(userId);
            entity.setCreateByName(userName);
            entity.setCreateTime(now);
        }
        entity.setModifyById(userId);
        entity.setModifyByName(userName);
        entity.setModifyTime(now);
    }
}
